%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                               %
% Implementation of the Kalman Filter algorithm                 %
% (Incremental Gauss-Newton) for solving a large                %
% system of linear equations in the least-squares               %
% sense                                                         %
%                                                               %
%                                                               %
%                                                               %
% INPUT :                                                       %
%        C,z,H,x = same meaning as in the book                  %
%        Cx =~ z   = one data block                             %
%        z         = (column vector)                            %
%        lamda     = 1 Kalman filter to solve exactely x = A\b  %
%        if 0 < lambda < 1 get real time Kalman filter          %
%        taille    = size of the problem                        %   
%                                                               %
%                                                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [H,x,testFailed] = kalman(C,z,H,x,lambda,taille, testFailed)

% disp('Kalman Entry');
[M,N]=size(C);

% Check for the size of the blocks

if N ~= taille
   error('Size problem in C');
end

% Initialization 

% An incremental algorithm can be used mainly in two ways:
% 1) To solve a large least-squares (lambda = 1)
% 2) To calibrate a model using real-time datas (0 < lambda < 1)
%
% In the first case, each data block has the same importance. 
% In the second, later blocks have more importance: at iteration j, block i
% (i<j) has ponderated weight lambda^(j-i).
% H(j+1) = lambda*H(j) + C'*C is the correct update formula according to
% lambda value, C being the block j

%-------------------------------------------------------------------------%
% update H here.....
    %TODO: H=......
%-------------------------------------------------------------------------%


% check if first matrix is singular, if yes, either change original matrix
% by small perturbation (H = H + delta*I) or change the order of the
% blocs...
while (det(H) <= 10^(-12))
   disp('Matrix H1 is singular... Update H = H + delta*I')
   % instead we could use H = H + 0.00001*eye(taille) to get non-singular
   % matrix...
   H = H + 0.05*eye(taille);
   % to use perturbated H, uncomment above line and comment the two
   % following ones...
   %testFailed = 1;
   %return;
                       % to ensure that the first H1 is a positive definite
                       % matrix. If we set Hzero to 0, the first computed
                       % H could be singular and cause numerical 
                       % difficulties
                       % another way to avoid singularity is to permute the
                       % order of the blocks...
end

%-------------------------------------------------------------------------%
% compute the next iterate x here.....
    %TODO: x=.......
%-------------------------------------------------------------------------%










